package provide ade_lib 1.0

proc create_aduser {dn upn sam pw} {
    new_object $dn;
    sof userPrincipalName $upn;
    sof sAMAccountName $sam;
    sof objectClass user;
    sof userAccountControl 514;
    save_object;
    append samAtDomain "$sam@" [domain_from_dn $dn]
    sup $samAtDomain $pw;
    sof userAccountControl 512;
    save_object;
}

proc create_adgroup {dn sam gtype} {
    new_object $dn;
    sof sAMAccountName $sam;
    sof objectClass group;
    switch $gtype {
        "global" {
                sof groupType [expr 0x80000002]
        }
        universal {
                sof groupType [expr 0x80000008]
        }
        local {
                sof groupType [expr 0x80000004]
        }
        default {
            error "invalid group type"
        }
    }

    save_object;
}

proc create_user {ad uname uid gid gecos home shell role} {

    set ztype [gzf type]
    if {$uname ne "-"
            || $uid ne "-"
            || $gid ne "-"
            || $gecos ne "-"
            || $home ne "-"
            || $shell ne "-"} {
        new_zone_user $ad;
        szuf uname $uname;
        szuf uid $uid;
        szuf gid $gid;
        # you cannot set the gecos field on classsic zones, so warn user and continue
        catch { szuf gecos $gecos;}
        szuf home $home;
        szuf shell $shell;
        if {[string match classic* $ztype]} {
            szuf enabled $role
        }
        save_zone_user;
    }
    if {[string match classic* $ztype] eq 0} {
        if {$role ne "-"} {
            if {[string first "+" $ad] == -1} {
                new_role_assignment $ad;
                set_role_assignment_field role $role;
                save_role_assignment;
            }
        }
    }
}

proc lremove {listVariable value} {
    upvar 1 $listVariable var
    set var [lsearch -not -inline -all -nocase $var $value]
}

proc remove_user_from_group {user group} {
    push
    set gdn [principal_to_dn $group]
    set udn [principal_to_dn $user]
    set udom [domain_from_dn $udn]
    set uforest [get_bind_info $udom forest]
    set gdom [domain_from_dn $gdn]
    set gforest [get_bind_info $gdom forest]
    if {$uforest != $gforest} {
         slo $udn
         set sid [gof sid]
         set udn "<SID="
         append udn $sid ">"
    }

    if {[catch {remove_object_value $gdn member $udn} err]} {
        puts "$err"
    }

    pop
}

proc add_user_to_group {user group} {
    push
    set gdn [principal_to_dn $group]
    set udn [principal_to_dn $user]
    set udom [domain_from_dn $udn]
    set uforest [get_bind_info $udom forest]
    set gdom [domain_from_dn $gdn]
    set gforest [get_bind_info $gdom forest]
    if {$uforest != $gforest} {
        slo $udn
        set sid [gof sid]
        set udn "<SID="
        append udn $sid ">"
    }

    if {[catch {add_object_value $gdn member $udn} err]} {
        puts "$err"
    }
    pop
}

proc create_assignment {adg role} {
    new_role_assignment $adg;
    set_role_assignment_field role $role;
    save_role_assignment;
}

proc create_group {adg name gid} {
    new_zone_group $adg;
    szgf name $name;
    szgf gid $gid;
    save_zone_group;
}

# Precreate a computer, create an zone computer or machine zone
# It will also create the AD computer object if necessary
# Similar to adjoin --precreate
#
# precreate_computer samaccount@domain ... 
# precreate_computer samaccount@domain ?-ad? ?-scp? ?-czone? ?-all? ?-container rdn? ?-dnsname dnsname? ?-trustee upn1? ?-trustee upn2? ...
#
# samaccount@domain contains both the computer name and the domain to join
# samaccount should be form of <computer>$, for example, the first argument can be paul-pc$pual.test
# Scp or computer zone will be created in current select zone, so need select_zone first
#
# computer:     computer account name
# Optional Arguments:
# -ad           (optional) Create AD computer object
#                          It will not create AD computer object if the computer object exists
# -scp          (optional) Precreate create zone computer(extension object)
#                          It will create AD computer object if not present
# -czone        (optional) Create computer zone
# -all          (optional) Create all above
#                          In addition, If no action specified, create ad and scp by default
# -container <rdn>
#               (optional) Subtree to create AD object
#                          This option takes effect only when a new AD computer object need to be created
#                          If empty, default to CN=Computers,DC=<domain>
# -dnsname <dnsname>
#               (optional) specified dns name
#                          If empty, derive the dnsname from samaccount name and domain
# -trustee <upn>
#               (optional) Arbitrary trustees who can use the precreated computers
#                          Note this option can be set multiple times
# Example:  Precreate computer "redhat" at default zone in centrify.com 
#           select_zone {CN=default,CN=Zones,DC=centrify,DC=com}
#           precreate_computer redhat$@centrify.com -trustee foo@centrify.com -trustee bar@child.centrify.com
proc precreate_computer {upn args} {
    set usage "precreate_computer samaccount@domain"
    set usage "$usage ?-ad? ?-scp? ?-czone? ?-all? ?-container rdn? ?-dnsname dnsname? ?-trustee upn1? ?-trustee upn2? ...";

    # Resolve samaccount@domain
    set dotIndex [string last "$@" $upn]
    set len [string length $upn]
    if {$dotIndex <= 0 || $dotIndex >= [expr $len-2]} {
        error "Computer name should be: samaccount@domain, and the samaccount should end with $"
    }
    set computer [string range $upn 0 [expr $dotIndex-1]]
    set domain [string range $upn [expr $dotIndex+2] $len]

    set base [dn_from_domain $domain]
    set sam "$computer\$"
    set pw [string tolower [string range $computer 0 13]]

    set createAD ""
    set createScp ""
    set createZone ""
    set explicitlyCreateAD ""
    set explicitlyCreateZone ""
    set trustees ""
    set container ""
    set dnsname ""
    set argCount [llength $args]
    for {set idx 0} {$idx < $argCount} {incr idx} {
        set flag [lindex $args $idx]
        switch -glob -- $flag {
            -ad {
               set createAD "true" 
               set explicitlyCreateAD "true"
            }
            -scp {
               set createScp "true" 
            }
            -czone {
               set createZone "true" 
               set explicitlyCreateZone "true"
            }
            -all {
               set createAD "true" 
               set createScp "true" 
               set createZone "true" 
            }
            -container {
                incr idx
                if {$idx >= $argCount} {
                    error "Missing -container argument: should be\n$usage"
                }
                set container [lindex $args $idx] 
            }
            -dnsname {
                incr idx
                if {$idx >= $argCount} {
                    error "Missing -dnsname argument: should be\n$usage"
                }
                set dnsname [lindex $args $idx] 
            }
            -trustee {
                incr idx
                if {$idx >= $argCount} {
                    error "Missing -trustee argument: should be\n$usage"
                }
                lappend trustees [lindex $args $idx] 
            }
            -* {
                error "Unknown flag $flag: should be\n$usage"
            }
            default {
                error "Unknown option $flag: should be\n$usage"
            }
        }
    }

    if {$createAD != "true" && $createScp != "true" && $createZone != "true"} {
        # Default to create ad and scp
        set createAD "true" 
        set createScp "true" 
    }

    set ztype ""
    set zone ""
    set zbase ""
    if {$createScp == "true" || $createZone == "true"} {
        set ztype [gzf type]
        if {$ztype != "tree" && $ztype != "classic3" &&  $ztype != "classic4"} {
            error "Wrong zone type $ztype, should be tree zone or classic zone"
        }
        set zone [gzf dn]
        set zbase [dn_from_domain [domain_from_dn $zone]]
    }
    if {$dnsname == ""} {
        # If not specified, dnsname should be computer.<joined domain>(same as console)
        set dnsname $computer.$domain
    }

    set addn ""
    if {$createAD == "true" || $createScp == "true"} {
        set objlist [get_objects -depth sub $base "(&(objectclass=computer)(cn=$computer))"]
        if {[llength $objlist] <= 0 && $createAD == "true"} {
            # If ad computer object does not exist and createAD
            # Create ad computer
            if {[string length $container] == 0} {
                set container computers
            }
            if {![catch {select_object $container}]} {
                set addn cn=$computer,$container
            } elseif {![catch {select_object $container,$base}]} {
                set addn cn=$computer,$container,$base
            } elseif {![catch {select_object cn=$container,$base}]} {
                set addn cn=$computer,cn=$container,$base
            } else {
                error "Invalid container"
            }

            new_object $addn
            sof objectclass computer 
            sof samaccountname $sam
            sof useraccountcontrol [expr 0x1000]
            svo
        } elseif {[llength $objlist] <= 0 && $createAD != "true"} { 
            # If ad computer object does not exist and not createAD
            # Error
            error "AD computer object doesn't exist, please specify -ad to create"
        } elseif {$explicitlyCreateAD == "true"} { 
            # If ad computer object exists
            # only error when explicitly creating AD computer object
            error "AD computer object already exists"
        } else {
            set addn [lindex $objlist 0]
        }

        # change computer password
        sup "$sam@$domain" $pw

        #enable computer account
        select_object $addn
        set control [gof userAccountControl]
        set control [expr $control & ~0x2]
        sof useraccountcontrol $control
        sof dNSHostName $dnsname
        set stypes "host http ftp cifs nfs"
        set spns ""
        foreach stype $stypes {
            lappend spns $stype/$computer
            lappend spns $stype/$dnsname
        }
        eval "sof servicePrincipalName $spns"
        svo
    }

    if {$createScp == "true"} {
        # deleting existing scp
        select_object $addn; 
        set sid [get_object_field sid]
        set filter "(|(keywords=parentLink:$sid)(managedBy=$addn))";
        foreach scp [get_objects -depth sub $zbase $filter] {
            select_object $scp; 
            delete_object;
        }

        new_zone_computer $sam@$domain
        set_zone_computer_field enabled false
        save_zone_computer

        select_object $addn; 
        set scpdn [get_objects -depth sub $zbase "(|(keywords=parentLink:[gof sid])(managedBy=$addn))"]
        if {[llength $scpdn] != 1} {
            error "Can not find the scp we just created, maybe replication problem."
        } else {
            set scpdn [lindex $scpdn 0]
        }

        set uacGuid [get_schema_guid User-Account-Control] 
        # Well-known User-Force-Change-Password
        set pwdGuid {00299570-246d-11d0-a768-00aa006e0529}
        # Well-known Validated-DNS-Host-Name
        set dhnGuid {72e39547-7b18-11d1-adef-00c04fd8d5cd}
        # Well-known Validated-SPN
        set spnGuid {f3a64788-5306-11d1-a9c5-0000f80367c1}
        foreach trustee $trustees {
            slo [principal_to_dn $trustee]
            set usid [gof sid]
            
            slo $scpdn
            set sd [gof sd]
            # Arbitrary Trustee, Zone/Computers/scp, This object, Generic Read
            set sd [add_sd_ace $sd "A;;GR;;;$usid"]
            sof sd $sd
            svo

            slo $addn
            set sd [gof sd]
            # Arbitrary Trustee, AD computer object, This object, This object, Reset Password
            set sd [add_sd_ace $sd "OA;;SW;$pwdGuid;;$usid"]
            # Arbitrary Trustee, AD computer object, This object, This object, Write userAccountControl
            set sd [add_sd_ace $sd "OA;;WP;$uacGuid;;$usid"]
            # Arbitrary Trustee, AD computer object, This object, This object, Validated write to DNS host name 
            set sd [add_sd_ace $sd "OA;;SW;$dhnGuid;;$usid"]
            # Arbitrary Trustee, AD computer object, This object, This object, Validated write to service principal name
            set sd [add_sd_ace $sd "OA;;SW;$spnGuid;;$usid"]
            sof sd $sd
            svo
        }

        set osGuid [get_schema_guid Operating-System] 
        set osvGuid [get_schema_guid Operating-System-Version]
        set oshGuid [get_schema_guid Operating-System-Hotfix]
        set osspGuid [get_schema_guid Operating-System-Service-Pack]
        slo $addn
        set csid [gof sid]
        set sd [gof sd]
        # SELF, AD computer object, This object, Write operatingSystem
        set sd [add_sd_ace $sd "OA;;WP;$osGuid;;PS"]
        # SELF, AD computer object, This object, Write operatingSystemVersion
        set sd [add_sd_ace $sd "OA;;WP;$osvGuid;;PS"]
        # SELF, AD computer object, This object, Write operatingSystemHotfix
        set sd [add_sd_ace $sd "OA;;WP;$oshGuid;;PS"]
        # SELF, AD computer object, This object, Write operatingSystemServicePack
        set sd [add_sd_ace $sd "OA;;WP;$osspGuid;;PS"]
        sof sd $sd
        svo

        set kwdGuid [get_schema_guid Keywords] 
        set disGuid [get_schema_guid Display-Name]
        slo $scpdn
        set sd [gof sd]
        # Precreated computer account, Zone/Computer/scp, This object, Generic Read
        set sd [add_sd_ace $sd "A;;GR;;;$csid"]
        # Precreated computer account, Zone/Computer/scp, This object, Write keywords property
        set sd [add_sd_ace $sd "OA;;WP;$kwdGuid;;$csid"]
        # Precreated computer account, Zone/Computer/scp, This object, Write displayName property
        set sd [add_sd_ace $sd "OA;;WP;$disGuid;;$csid"]
        sof sd $sd
        svo
    }

    if {$createZone == "true"} {
        append MZ $dnsname@$zone

        # check whether tree zone
        if {$ztype == "tree"} {
            set tcl_interactive 0
            catch {select_zone $MZ;dlz}
            create_zone computer $MZ std
            
            select_zone $MZ
            set authdn "CN=Authorization,$zone"
            set azadGuid [get_schema_guid ms-DS-Az-Application-Data]
            foreach trustee $trustees {
                # Arbitrary Trustee, <computer-scope>/AzRoleObjectContainer, This object, Generic Read
                # Arbitrary Trustee, <computer-scope>/AzRoleObjectContainer, This object, Create msDS-AzRole objects
                # Arbitrary Trustee, <computer-scope>/AzRoleObjectContainer, This object, delete msDS-AzRole objects
                # Arbitrary Trustee, <computer-scope>/AzRoleObjectContainer, msDS-AzRole objects, write msDS-AzApplicationData property
                # Arbitrary Trustee, computer:zone/Users
                delegate_zone_right add_user $trustee
                delegate_zone_right change_user $trustee
                delegate_zone_right delete_user $trustee
                # Arbitrary Trustee, computer:zone/Groups
                delegate_zone_right add_group $trustee
                delegate_zone_right change_group $trustee
                delegate_zone_right delete_group $trustee

                slo [principal_to_dn $trustee]
                set usid [gof sid]

                slo $authdn
                set sd [gof sd]
                # Arbitrary Trustee, Zone/Authorization, This object, Generic Read
                set sd [add_sd_ace $sd "A;;GR;;;$usid"]
                # Arbitrary Trustee, Zone/Authorization, This object, write msDS-AzApplicationData property
                set sd [add_sd_ace $sd "OA;;WP;$azadGuid;;$usid"]
                sof sd $sd
                svo
            }

            # restore current zone
            select_zone $zone
        } elseif {$explicitlyCreateZone == "true"} { 
            # only error when explicitly creating machine zone
            error "Wrong zone type $ztype, should be tree zone"
        }
    }
}

proc getopt {_argv name {_var ""} } {
    upvar 1 $_argv argv $_var var
    set pos [lsearch -regexp $argv ^$name]
    if {$pos>=0} {
        set to $pos
        if {$_var ne ""} {
            set var [lindex $argv [incr to]]
        }
        set argv [lreplace $argv $pos $to]
        return 1
    } else {
        return 0
    }
}

proc convert_msdate {msdate} {
    if {$msdate==9223372036854775807} {
        return -1
    }
    return [clock format [expr ($msdate/10000000)-11644473600]]
}

set dict_gt [dict create]
dict set dict_gt 0x0001 "SYSTEM" 
dict set dict_gt 0x0002 "GLOBAL" 
dict set dict_gt 0x0004 "DOMAIN_LOCAL" 
dict set dict_gt 0x0008 "UNIVERSAL" 
dict set dict_gt 0x0010 "APP_BASIC" 
dict set dict_gt 0x0020 "APP_QUEEY" 
dict set dict_gt 0x80000000 "SECURITY" 

proc explain_groupType {gt} {
    global dict_gt
    set r [format "%x" $gt]
    dict for {k v} $dict_gt {
        if [expr $gt&$k] {
            lappend r $v
	    }
    }
    return $r
}

set dict_ta [dict create]
dict set dict_ta 0x0001 "NON_TRANSITIVE" 
dict set dict_ta 0x0002 "UPLEVEL_ONLY" 
dict set dict_ta 0x0004 "QUARANTINED_DOMAIN" 
dict set dict_ta 0x0008 "FOREST_TRANSITIVE" 
dict set dict_ta 0x0010 "CROSS_ORGANIZATION" 
dict set dict_ta 0x0020 "WITHIN_FOREST" 
dict set dict_ta 0x0040 "TREAT_AS_EXTERNAL" 
dict set dict_ta 0x0080 "USE_RC4_ENCRYPTION" 

proc explain_trustAttributes {ta} {
    global dict_ta
    set r [format "%x" $ta]
    dict for {k v} $dict_ta {
        if [expr $ta&$k] {
            lappend r $v
        }
    }
    return $r
}

proc explain_trustDirection {td} {
    switch $td {
	0 {return "disabled"}
	1 {return "inbound"}
	2 {return "outbound"}
	3 {return "two-way"}
        default {return "unknown"}
    }
}

set dict_uac [dict create]
dict set dict_uac 0x0001 "ADS_UF_SCRIPT" 
dict set dict_uac 0x0002 "ADS_UF_ACCOUNTDISABLE" 
dict set dict_uac 0x0008 "ADS_UF_HOMEDIR_REAUIRED" 
dict set dict_uac 0x0010 "ADS_UF_LOCKOUT" 
dict set dict_uac 0x0020 "ADS_UF_PASSWD_BITREQD" 
dict set dict_uac 0x0040 "ADS_UF_PASSWD_CANT_CHANGE" 
dict set dict_uac 0x0080 "ADS_UF_ENCRYPTED_TEXT_PASSWORD_ALLOWED" 
dict set dict_uac 0x0100 "ADS_UF_TEMP_DUPLICATE_ACCOUNT" 
dict set dict_uac 0x0200 "ADS_UF_NORMAL_ACCOUNT" 
dict set dict_uac 0x0800 "ADS_UF_INTERDOMAIN_TRUST_ACCOUNT" 
dict set dict_uac 0x1000 "ADS_UF_WORKSTATION_TRUST_ACCOUNT" 
dict set dict_uac 0x2000 "ADS_UF_SERVER_TRUST_ACCOUNT" 
dict set dict_uac 0x010000 "ADS_UF_DONT_EXPIRE_PASSWD" 
dict set dict_uac 0x020000 "ADS_UF_MNS_LOGON_ACCOUNT" 
dict set dict_uac 0x040000 "ADS_UF_SMARTCARD_REQUIRED" 
dict set dict_uac 0x080000 "ADS_UF_TRUSTED_FOR_DELEGATION" 
dict set dict_uac 0x100000 "ADS_UF_NOT_DELEGATED" 
dict set dict_uac 0x200000 "ADS_UF_USE_DES_KEY_ONLY" 
dict set dict_uac 0x400000 "ADS_UF_DONT_REQUIRE_PREAUTH" 
dict set dict_uac 0x800000 "ADS_UF_PASSWORD_EXPIRED" 

proc explain_userAccountControl {uac} {
    global dict_uac
    set r [format "%x" $uac]
    dict for {k v} $dict_uac {
        if [expr $uac&$k] {
            lappend r $v
	    }
    }
    return $r
}

proc list_zones {domain} {
    push
    foreach  zone [get_zones $domain] {
	if {[catch {select_zone -nc $zone}]} {
            puts "Cannot open zone: $zone"
        } else {
            puts "$zone : [gzf type] : [gzf schema] " }
    }
    pop
}

# Modify a timebox string (42 hex characters)
# strTimeBox:   original timebox string(42 hex characters)
# day:          the day of week to modify(Sunday=0)
# hour:         the hour to modify(0-23)
#               0 means 0:00~1:00, 23 means 23:00~24:00
# avail:        0 - make this hour unavailable
#               not 0 - make this hour available
# Example:  Let Saturday 23:00~24:00(Sa: 23:00-00:00) available
#           set tb 000000000000000000000000000000000000000000
#           puts [modify_timebox $tb 6 23 1]
#           800000000000000000000000000000000000000000
proc modify_timebox {strTimeBox day hour avail} {
    # Trim the head 0 in hour
    set hour [string trimleft $hour 0]
    if {$hour == ""} {
        set hour 0
    }
    if {$hour < 0 || $hour > 23} {
        error "Invalid hour: $hour. Should be 0~23"
    }
    if {$day < 0 || $day > 6} {
        error "Invalid day of week: $day. Should be 0~6"
    }
    set dayIndex [expr 6*int($day)+2]
    # 4 bit a hex characters, each bit represents one hour
    set hourIndex [expr int($hour/4)]
    # Because the timebox string is little-ending, and the higher bit is bigger hour
    # So change 0 1 2 3 4 5 -> 1 0 3 2 5 4
    set hourIndex [expr int($hourIndex/2)*4+1-$hourIndex] 
    set hourMask [expr int(pow(2, int($hour)%4))]
    # calculate index char 
    set i [expr ($dayIndex+$hourIndex)%42]
    # get original value
    set v 0x[string index $strTimeBox $i]
    # modify value
    if {$avail == "0"} {
        set v [format "%X" [expr $v & ~$hourMask]]
    } else {
        set v [format "%X" [expr $v | $hourMask]]
    }
    return [string replace $strTimeBox $i $i $v]
}
