#!/bin/sh /usr/share/centrifydc/perl/run

use strict;
use lib '/usr/share/centrifydc/perl';

use CentrifyDC::GP::Mapper;

my $file;
my $action;
my $user;
$file = {
    'comment_markers' => [
      '#',
    ],
    'hierarchy_separator' => '.',
    'list_expr' => ', *| +',
    'list_separator' => ', ',
    'lock' => '/etc/centrifydc/centrifydc.conf.lock',
    'match_expr' => [
      '/^\s*([^\s:=]+)[:=]\s*(.*)/',
    ],
    'named_list_separator' => ',',
    'parent_expr' => '^(.*)\.([^\.]+)$',
    'path' => [
      '/etc/centrifydc/centrifydc.conf',
    ],

    'value_map' => {

      'fips.mode.enable' => {
        'default_data' => 'false',
        'post_action' => [
          'DO_RESTART_ADCLIENT_AND_EXPIRE_CACHE',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDC/Settings/Fips',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'fips.mode.enable',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },

    },

    'write_data' => '$value: $data\n',
};

sub IsSupported();



#
# check if current OS supports FIPS. Only Mac and RedHat Linux supports FIPS.
#
#   return: 1       - FIPS supported
#           0       - FIPS not supported
#           undef   - failed
#
sub IsFipsSupported()
{
    my $supported = 0;

    my $system = `uname -s`;
    chomp $system;

    if ($system eq 'Darwin')
    {
        $supported = 1;
    }
    elsif ($system eq 'Linux')
    {
        if (-x '/usr/bin/lsb_release')
        {
            # first try to identify the system using lsb
            my $release = `/usr/bin/lsb_release -i`;
            chomp $release;

            if ($release =~ m/RedHat/)
            {
                $supported = 1;
            }
        }
    }

    return $supported;
}



$action = $ARGV[0];
my $mode = $ARGV[2] ? $ARGV[2] : $ARGV[1];
$user = $ARGV[2] ? $ARGV[1] : undef;

IsFipsSupported() or exit(0);

if ($action eq "unmap")
{

    CentrifyDC::GP::Mapper::UnMap($file, $user);
}
else
{
    CentrifyDC::GP::Mapper::Map($file, $user);
}
