#!/bin/sh /usr/share/centrifydc/perl/run
#
# Copyright (C) 2004-2008 Centrify Corporation.
#
# Machine/user mapper script that updates crontab entries.
#

use strict;

use lib '/usr/share/centrifydc/perl';

use CentrifyDC::GP::Args;
use CentrifyDC::GP::General qw(:debug ReadFile RunCommand);
use CentrifyDC::GP::Lock;
use CentrifyDC::GP::Registry;                                             


sub CrontabEntry($$);

my $registrykey = "software/policies/centrify/unixsettings/crontabentries";
my %regVar;
my $key;

my $args = CentrifyDC::GP::Args->new();
my $action = $args->action();

CentrifyDC::GP::Registry::Load($args->user());

# filename of our temporary output file
my $newfilename = "/tmp/temp.out.crontab";

# Identify operating system
my $uname=`uname -s`;
my $CRON_LIST="su %s -c 'crontab -l' 2>/dev/null |";
my $CRON_CREATE="su %s -c 'crontab $newfilename' >/dev/null 2>&1 ; rm $newfilename";
my $CRON_REMOVE="crontab -r %s >/dev/null 2>&1 ";
my $CRON_ALLOW="/usr/lib/cron/cron.allow";
my $CRON_DENY="/usr/lib/cron/cron.deny";
$uname =~ s/[\s|\t|\r|\f|\n]//;
if ("$uname" eq "Linux")
{
  $CRON_LIST="crontab -u %s -l 2>/dev/null |";
  $CRON_CREATE="crontab -u %s $newfilename >/dev/null 2>&1; rm $newfilename";
  $CRON_REMOVE="crontab -u %s -r >/dev/null 2>&1 ";
  $CRON_ALLOW="/etc/cron.allow";
  $CRON_DENY="/etc/cron.deny";
  if (`uname -r` =~ m/(2\.4\.19\-.*|2\.4\.21\-198.*|2\.6\.4-5.*|2\.6\.5\-7.*)/)
  {
    $CRON_ALLOW="/var/spool/cron/allow";
    $CRON_DENY="/var/spool/cron/deny";
  }
}
else
{
 if ("$uname" eq "Darwin")
  {
   $CRON_REMOVE="crontab -u %s -r >/dev/null 2>&1 ";
    `/usr/bin/sw_vers -productVersion` =~ m/(10\.\d)\.?(\d+)?/;
    if ($1 eq "10.4")
    {
      $CRON_ALLOW="/var/cron/allow";
      $CRON_DENY="/var/cron/deny";
  }
  }
 my $AIX_34=`uname -rv`;
 $AIX_34=~ s/[\n]//;
 if ("$AIX_34" eq "3 4")
  {
    $CRON_REMOVE="su %s -c 'crontab -r' >/dev/null 2>&1 ";
  }
}


# we will always clean up the crontab which is between the markerstart and markerend.
#  method: find all the user at the previous policy file, clean up the crontab,
#    then rewrite the value which get from the Registery.pol
#
my @keys = CentrifyDC::GP::Registry::Values($args->class(), $registrykey, "previous");
if (defined($keys[0]))
{
    foreach $key(@keys) {
        my @values = CentrifyDC::GP::Registry::Query($args->class(), $registrykey,
            "previous", $key);
    my $croncommand = $values[1];
    $croncommand =~ s/^\s*//;
    my @entry = split(/ +/, $croncommand);
    if ($values[1]) {
            DEBUG_OUT("Previous policy file crontab entry: $values[1]");
     }
    # Min Hour DayOfMonth Month DayOfWeek User command
    next if (scalar(@entry) < 7);
    my $cuser = $entry[5];
    #when we remove the relative crontab before, we set the $action='unmap'
    $action="unmap";
    #it just remove $cuser's crontab
    CrontabEntry($cuser, "");
    }
    $action="map";
}

# begin to get the present values to write the crontab
@keys = CentrifyDC::GP::Registry::Values($args->class(), $registrykey, "current");
if (defined($keys[0]))
{
    foreach $key(@keys) {
        my @values = CentrifyDC::GP::Registry::Query($args->class(), $registrykey,
            "current", $key);
        $regVar{$key} = $values[1];
        DEBUG_OUT("Current policy file crontab entry: $values[1]");
    } # foreach
} # if

my @cusers;
my @cronentries;
my %cronvalues;



# crontab entries by user
foreach my $key (sort(keys %regVar)) {
    # skip comments and blank lines
    next if ($regVar{$key} =~ /^[ \t]*\#/o);
    next if ($regVar{$key} =~ /^[ \t]*$/o);

    # remove the space in the beginning of the crontab command
    my $croncommand = $regVar{$key};
    $croncommand =~ s/^\s*//;
    my @entry = split(/ +/, $croncommand);    
    # Min Hour DayOfMonth Month DayOfWeek User command
    next if (scalar(@entry) < 7);
    # get the user
    my $cuser = $entry[5];
    splice(@entry, 5, 1);
    # the same user's command, we will put together
    push(@{$cronvalues{$cuser}},"\n@entry\n");
} # foreach

foreach my $cuser (keys %cronvalues) {
  CrontabEntry($cuser, "@{$cronvalues{$cuser}}");
  DEBUG_OUT("Write crontab entry: @{$cronvalues{$cuser}} for user $cuser");
}

# write the crontab for each user
sub CrontabEntry($$)
{
    my ($cuser, $command) = @_;
    open NEWFILE, ">$newfilename" or die;
    my $LIST_COMMAND=sprintf($CRON_LIST, $cuser);
    my $CREATE_COMMAND=sprintf($CRON_CREATE, $cuser);
    my $REMOVE_COMMAND=sprintf($CRON_REMOVE, $cuser);
    my $empty = 1;

    # search through the config file to find the key from the registry file
    open CONFIG, "$LIST_COMMAND";
    my $markerstart = "#** Generated via group policy by centrify for $cuser Start **\n";
    my $markerend   = "\n#** Generated via group policy by centrify for $cuser End   **\n";
    my $configline;
    my $found = 0;
    while ($configline = <CONFIG>) {
        if ($configline eq $markerstart) {
            $found = 1;
            if ($action eq "map") {
                print NEWFILE $markerstart; 
                print NEWFILE $command;
                print NEWFILE $markerend;
                $empty = 0;
            } # if

            my $cl;
            while (($cl = <CONFIG>) and ($cl ne $markerend)) {
                # do nothing
            } # while
        } # if
        else {
           # do not copy the warning lines, they get regenerated by the crontab command
	    next if ($configline =~ /^\# DO NOT EDIT THIS FILE/o);
            next if ($configline =~ /^\# \(\/tmp\/temp.out.crontab/o);
    	    next if ($configline =~ /^\# \(Cron versio/o);

            if ($configline eq $markerend) {
                # error condition
            } # if
            else {
                print NEWFILE $configline; 
                $empty = 0;
            } # else

            # if the same exact line already exists in our configfile,
            # we don't need to add it again
            chomp($configline);
        } # else
    } # while

    if (!$found and ($action eq "map")) {
        print NEWFILE $markerstart;
        print NEWFILE $command;
        print NEWFILE $markerend; # we didn't find a CIMS section, so we'll tack it on at the end.
        $empty = 0;
    } # if
    close NEWFILE;
    close CONFIG;
    chmod 0777, $newfilename;

    # If there is no entry for the crontab, we should remove the crontab file completely.
    if ($empty)
    {
        DEBUG_OUT("Remove empty crontab file");
        my $rc = RunCommand($REMOVE_COMMAND);
        if (! defined($rc) or $rc ne '0')
        {
            ERROR_OUT("Cannot remove empty crontab file");
        }
        return;
    }

    #before create the crontab, we need check the cron.allow or allow file
    my $content;
    if (-e $CRON_ALLOW) {
        $content=ReadFile($CRON_ALLOW);
        if ($content !~ m|$cuser$|ms)
        {
            my $lock = CentrifyDC::GP::Lock->new($CRON_ALLOW);
            if (! defined($lock))
            {
                FATAL_OUT("Cannot obtain lock for $CRON_ALLOW");
            }
            open(ALLOW, ">>$CRON_ALLOW");
            print (ALLOW "$cuser\n");
            close ALLOW;

            $content=ReadFile($CRON_ALLOW);
        }
    } else {
        if (-e $CRON_DENY) {
            $content=ReadFile($CRON_DENY);
            if ($content =~ m|$cuser$|ms)
            {
                my $lock = CentrifyDC::GP::Lock->new($CRON_DENY);
                if (! defined($lock))
                {
                    FATAL_OUT("Cannot obtain lock for $CRON_DENY");
                }
                open(DENY, ">$CRON_DENY");
                $content =~ s/$cuser\n//ms;
                print (DENY "$content");
                close DENY;
            }
        }
    }

    DEBUG_OUT("Create crontab entry using command [$CREATE_COMMAND]");
    my $rc = RunCommand($CREATE_COMMAND, "/tmp/createcrontab.lock");
    if (! defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot create crontab entry");
    }
}

